import os
try:
    import fastentrypoints
except ImportError:
    import sys
    print("fastentrypoints module not found. entry points will be slower.", file=sys.stderr)
from setuptools import find_packages, setup
from setuptools.command.install import install

from babel.messages import frontend as babel


def get_assets(name):
    """
    Recursive get dirs from middlewared/{name}
    """
    base_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'middlewared',
    )
    result = []
    for root, dirs, files in os.walk(os.path.join(base_path, name)):
        result.extend([f'{os.path.relpath(root, base_path)}/*'] + [
            os.path.join(os.path.relpath(root, base_path), file)
            for file in filter(lambda f: f == '.gitkeep', files)
        ])
    return result


class InstallWithBabel(install):
    def run(self):
        compiler = babel.compile_catalog(self.distribution)
        option_dict = self.distribution.get_option_dict('compile_catalog')
        compiler.domain = [option_dict['domain'][1]]
        compiler.directory = option_dict['directory'][1]
        compiler.run()
        super().run()


setup(
    name='middlewared',
    description='TrueNAS Middleware Daemon',
    packages=find_packages(),
    package_data={
        'middlewared.apidocs': [
            'templates/websocket/*',
            'templates/*.*',
        ],
        'middlewared': (
            get_assets('alembic') +
            ['alembic.ini'] +
            get_assets('assets') +
            get_assets('etc_files') +
            get_assets('migration') +
            get_assets('plugins/kubernetes_linux/migrations') +
            get_assets('plugins/kubernetes_linux/app_migrations')
        ),
    },
    include_package_data=True,
    license='BSD',
    platforms='any',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    entry_points={
        'console_scripts': [
            'configure_fips = middlewared.scripts.configure_fips:main',
            'setup_cgroups = middlewared.scripts.setup_cgroups:main',
            'middlewared = middlewared.main:main',
            'midclt = middlewared.client.client:main',
            'midgdb = middlewared.scripts.gdb:main',
            'sedhelper = middlewared.scripts.sedhelper:main',
            'wait_to_hang_and_dump_core = middlewared.scripts.wait_to_hang_and_dump_core:main',
            'wait_on_disks = middlewared.scripts.wait_on_disks:main',
        ],
    },
    cmdclass={
        'install': InstallWithBabel,

        'compile_catalog': babel.compile_catalog,
        'extract_messages': babel.extract_messages,
        'init_catalog': babel.init_catalog,
        'update_catalog': babel.update_catalog,
    },
)
